@@ -19,11 +19,17 @@ from justbytes import Range
 from dateutil import parser as date_parser
 
 from .._errors import StratisCliEngineError
+from .._errors import StratisCliIncoherenceError
+from .._errors import StratisCliNoChangeError
+from .._errors import StratisCliPartialChangeError
 
 from .._stratisd_constants import StratisdErrors
 
 from ._connection import get_object
+from ._constants import FILESYSTEM_INTERFACE
 from ._constants import TOP_OBJECT
+from ._formatting import TABLE_FAILURE_STRING
+from ._formatting import fetch_property
 from ._formatting import print_table
 
 
@@ -38,9 +44,15 @@ class LogicalActions:
         Create volumes in a pool.
 
         :raises StratisCliEngineError:
+        :raises StratisCliIncoherenceError:
+        :raises StratisCliPartialChangeError:
         """
+        # pylint: disable=too-many-locals
+        # pylint: disable=import-outside-toplevel
+        from ._data import MOFilesystem
         from ._data import ObjectManager
         from ._data import Pool
+        from ._data import filesystems
         from ._data import pools
 
         proxy = get_object(TOP_OBJECT)
@@ -51,18 +63,45 @@ class LogicalActions:
             .search(managed_objects)
         )
 
-        (_, rc, message) = Pool.Methods.CreateFilesystems(
-            get_object(pool_object_path), {"specs": namespace.fs_name}
+        requested_names = frozenset(namespace.fs_name)
+
+        names = frozenset(
+            MOFilesystem(info).Name()
+            for (_, info) in filesystems(props={"Pool": pool_object_path}).search(
+                managed_objects
+            )
+        )
+        already_names = requested_names.intersection(names)
+
+        if already_names != frozenset():
+            raise StratisCliPartialChangeError(
+                "create", requested_names.difference(already_names), already_names
+            )
+
+        ((created, list_created), rc, message) = Pool.Methods.CreateFilesystems(
+            get_object(pool_object_path), {"specs": list(requested_names)}
         )
 
         if rc != StratisdErrors.OK:  # pragma: no cover
             raise StratisCliEngineError(rc, message)
 
+        if not created or len(list_created) < len(requested_names):  # pragma: no cover
+            raise StratisCliIncoherenceError(
+                (
+                    "Expected to create the specified filesystems in pool %s "
+                    "but stratisd reports that it did not actually create "
+                    "some or all of the filesystems requested"
+                )
+                % namespace.pool_name
+            )
+
     @staticmethod
     def list_volumes(namespace):
         """
         List the volumes in a pool.
         """
+        # pylint: disable=import-outside-toplevel
+        from ._data import FetchProperties
         from ._data import MOFilesystem
         from ._data import MOPool
         from ._data import ObjectManager
@@ -76,9 +115,12 @@ class LogicalActions:
         proxy = get_object(TOP_OBJECT)
         managed_objects = ObjectManager.Methods.GetManagedObjects(proxy, {})
 
-        mofilesystems = (
-            MOFilesystem(info)
-            for _, info in filesystems(
+        filesystems_with_props = [
+            (
+                FetchProperties.Methods.GetAllProperties(get_object(objpath), {}),
+                MOFilesystem(info),
+            )
+            for objpath, info in filesystems(
                 props=None
                 if pool_name is None
                 else {
@@ -89,7 +131,7 @@ class LogicalActions:
                     )[0]
                 }
             ).search(managed_objects)
-        )
+        ]
 
         path_to_name = dict(
             (path, MOPool(info).Name())
@@ -98,18 +140,36 @@ class LogicalActions:
             ).search(managed_objects)
         )
 
+        def filesystem_used(props):
+            """
+            Calculate the string value to display for filesystem used.
+
+            The format is just that chosen by justbytes default configuration.
+
+            :param props: a dictionary of property values obtained
+            :type props: a dict of str * object
+            :returns: a string to display in the resulting list output
+            :rtype: str
+            """
+            filesystem_used = fetch_property(FILESYSTEM_INTERFACE, props, "Used", Range)
+            return (
+                TABLE_FAILURE_STRING
+                if filesystem_used is None
+                else str(filesystem_used)
+            )
+
         tables = [
-            [
+            (
                 path_to_name[mofilesystem.Pool()],
                 mofilesystem.Name(),
-                str(Range(mofilesystem.Used())),
+                filesystem_used(props),
                 date_parser.parse(mofilesystem.Created())
                 .astimezone()
                 .strftime("%b %d %Y %H:%M"),
                 mofilesystem.Devnode(),
                 mofilesystem.Uuid(),
-            ]
-            for mofilesystem in mofilesystems
+            )
+            for props, mofilesystem in filesystems_with_props
         ]
 
         print_table(
@@ -124,7 +184,12 @@ class LogicalActions:
         Destroy volumes in a pool.
 
         :raises StratisCliEngineError:
+        :raises StratisCliIncoherenceError:
+        :raises StratisCliPartialChangeError:
         """
+        # pylint: disable=too-many-locals
+        # pylint: disable=import-outside-toplevel
+        from ._data import MOFilesystem
         from ._data import ObjectManager
         from ._data import Pool
         from ._data import filesystems
@@ -138,28 +203,55 @@ class LogicalActions:
             .require_unique_match(True)
             .search(managed_objects)
         )
+
+        requested_names = frozenset(namespace.fs_name)
+
+        pool_filesystems = {
+            MOFilesystem(info).Name(): op
+            for (op, info) in filesystems(props={"Pool": pool_object_path}).search(
+                managed_objects
+            )
+        }
+        already_removed = requested_names.difference(frozenset(pool_filesystems.keys()))
+
+        if already_removed != frozenset():
+            raise StratisCliPartialChangeError(
+                "destroy", requested_names.difference(already_removed), already_removed
+            )
+
         fs_object_paths = [
-            op
-            for name in namespace.fs_name
-            for (op, _) in filesystems(
-                props={"Name": name, "Pool": pool_object_path}
-            ).search(managed_objects)
+            op for (name, op) in pool_filesystems.items() if name in requested_names
         ]
 
-        (_, rc, message) = Pool.Methods.DestroyFilesystems(
+        ((destroyed, list_destroyed), rc, message) = Pool.Methods.DestroyFilesystems(
             get_object(pool_object_path), {"filesystems": fs_object_paths}
         )
 
         if rc != StratisdErrors.OK:  # pragma: no cover
             raise StratisCliEngineError(rc, message)
 
+        if not destroyed or len(list_destroyed) < len(
+            # pylint: disable=bad-continuation
+            fs_object_paths
+        ):  # pragma: no cover
+            raise StratisCliIncoherenceError(
+                (
+                    "Expected to destroy the specified filesystems in pool %s "
+                    "but stratisd reports that it did not actually destroy "
+                    "some or all of the filesystems requested"
+                )
+                % namespace.pool_name
+            )
+
     @staticmethod
     def snapshot_filesystem(namespace):
         """
         Snapshot filesystem in a pool.
 
         :raises StratisCliEngineError:
+        :raises StratisCliNoChangeError:
         """
+        # pylint: disable=import-outside-toplevel
         from ._data import ObjectManager
         from ._data import Pool
         from ._data import filesystems
@@ -179,7 +271,7 @@ class LogicalActions:
             .search(managed_objects)
         )
 
-        (_, rc, message) = Pool.Methods.SnapshotFilesystem(
+        ((changed, _), rc, message) = Pool.Methods.SnapshotFilesystem(
             get_object(pool_object_path),
             {"origin": origin_fs_object_path, "snapshot_name": namespace.snapshot_name},
         )
@@ -187,11 +279,18 @@ class LogicalActions:
         if rc != StratisdErrors.OK:  # pragma: no cover
             raise StratisCliEngineError(rc, message)
 
+        if not changed:
+            raise StratisCliNoChangeError("snapshot", namespace.snapshot_name)
+
     @staticmethod
     def rename_fs(namespace):
         """
         Rename a filesystem.
+
+        :raises StratisCliEngineError:
+        :raises StratisCliNoChangeError:
         """
+        # pylint: disable=import-outside-toplevel
         from ._data import ObjectManager
         from ._data import Filesystem
         from ._data import filesystems
@@ -210,9 +309,12 @@ class LogicalActions:
             .search(managed_objects)
         )
 
-        (_, rc, message) = Filesystem.Methods.SetName(
+        ((changed, _), rc, message) = Filesystem.Methods.SetName(
             get_object(fs_object_path), {"name": namespace.new_name}
         )
 
         if rc != StratisdErrors.OK:  # pragma: no cover
             raise StratisCliEngineError(rc, message)
+
+        if not changed:
+            raise StratisCliNoChangeError("rename", namespace.new_name)
